/* 
 *
 * $Id: k3bsetup.h,v 1.9 2003/06/12 14:07:41 trueg Exp $
 *
 * This file is part of the K3b project.
 * Copyright (C) 1998-2003 Sebastian Trueg <trueg@k3b.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */


#ifndef K3BSETUP_H
#define K3BSETUP_H


class KConfig;

#include <qobject.h>
#include <qstring.h>
#include <qstringlist.h>


class K3bExternalBinManager;
class KSimpleConfig;
namespace K3bCdDevice {
  class CdDevice;
  class DeviceManager;
}


/**
 * K3bSetup represents the setup status.
 */
class K3bSetup : public QObject
{
Q_OBJECT

 public:
  K3bSetup( QObject* = 0 );
  ~K3bSetup();

  bool saveConfig();

  void setApplyDevicePermissions( bool b ) { m_applyDevicePermissions = b; }
  void setApplyExternalBinPermissions( bool b ) { m_applyExternalBinPermission = b; }
  void setCreateFstabEntries( bool b ) { m_createFstabEntries = b; }

  bool applyExternalProgramPermissions() const { return m_applyExternalBinPermission; }
  bool applyDevicePermissions() const { return m_applyDevicePermissions; }
  bool createFstabEntries() const { return m_createFstabEntries; }

  void setCdWritingGroup( const QString& );
  void addUser( const QString& );
  void clearUsers();

  const QString& cdWritingGroup() const;
  const QStringList& users() const;

  K3bCdDevice::DeviceManager* deviceManager() const { return m_deviceManager; }
  K3bExternalBinManager* externalBinManager() const { return m_externalBinManager; }

 signals:
  void writingSettings();
  void writingSetting( const QString& s );
  void settingWritten( bool success, const QString& comment );
  void error( const QString& error );

 private:
  uint createCdWritingGroup();
  void doApplyExternalProgramPermissions( uint );
  void doApplyDevicePermissions( uint );
  void doCreateFstabEntries();

  QString m_cdwritingGroup;
  QStringList m_userList;

  K3bCdDevice::DeviceManager* m_deviceManager;
  K3bExternalBinManager* m_externalBinManager;

  bool m_applyDevicePermissions;
  bool m_applyExternalBinPermission;
  bool m_createFstabEntries;

  QString m_configPath;
  KSimpleConfig* m_config;

  friend class FstabEntriesTab;
};

#endif
